/*
 * check-name: find interesting items for symbol
 * check-command: undertaker -j checkexpr -m ../kconfig-dumps/models/x86.model CONFIG_SND_HDA_INTEL
 * check-output-start
I: loaded rsf model for x86
I: Using x86 as primary model
CONFIG_64BIT=n
CONFIG_ARCH_SUPPORTS_MSI=y
CONFIG_FUTEX=y
CONFIG_HAS_IOMEM=y
CONFIG_I2C=y
CONFIG_I2C_MODULE=n
CONFIG_INPUT=y
CONFIG_INPUT_MODULE=n
CONFIG_MODULES=y
CONFIG_PCI=y
CONFIG_RT_MUTEXES=y
CONFIG_SMP=y
CONFIG_SND=y
CONFIG_SND_AC97_CODEC=y
CONFIG_SND_AC97_CODEC_MODULE=n
CONFIG_SND_CA0106=y
CONFIG_SND_CA0106_MODULE=n
CONFIG_SND_HDA_INTEL=y
CONFIG_SND_HDA_INTEL_MODULE=n
CONFIG_SND_ICE1724=y
CONFIG_SND_ICE1724_MODULE=n
CONFIG_SND_MODULE=n
CONFIG_SND_PCI=y
CONFIG_SND_POWERMAC=n
CONFIG_SND_POWERMAC_MODULE=n
CONFIG_SND_PPC=n
CONFIG_SND_VMASTER=y
CONFIG_SOUND=y
CONFIG_SOUND_MODULE=n
CONFIG_X86_32=y
CONFIG_X86_32_NON_STANDARD=y
CONFIG_X86_64=n
CONFIG_X86_EXTENDED_PLATFORM=y
CONFIG_X86_IO_APIC=y
CONFIG_X86_LOCAL_APIC=y
CONFIG_X86_UP_APIC=n
# M68K=n
# NO_IOMEM=n
# PPC=n
# PPC_PMAC=n
# S390=n
# n=n
 * check-output-end
 */
