#if ENABLE_FEATURE_HTTPD_USE_SENDFILE
#endif

#ifndef PIPE_BUF
#endif

#if PIPE_BUF >= IOBUF_SIZE
#endif

#if ENABLE_FEATURE_HTTPD_BASIC_AUTH
#endif

#if ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR
#endif

#if ENABLE_FEATURE_HTTPD_ERROR_PAGES
#endif

#if ENABLE_FEATURE_HTTPD_PROXY
#endif

#if ENABLE_FEATURE_HTTPD_RANGES
#else
#endif

#if ENABLE_FEATURE_HTTPD_GZIP
#else
#endif

#if ENABLE_FEATURE_HTTPD_ENCODE_URL_STR
#endif

#if ENABLE_FEATURE_HTTPD_CGI
#    if ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV
#    endif
#    if ENABLE_FEATURE_HTTPD_BASIC_AUTH
#    endif
#    if ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR
#    endif
#endif /* FEATURE_HTTPD_CGI */

#if DEBUG
#endif

#if ENABLE_FEATURE_HTTPD_CGI
#elif ENABLE_FEATURE_HTTPD_PROXY
#endif

#if ENABLE_FEATURE_HTTPD_CGI
#else
#endif

#if BB_MMU
#else
#endif

#if ENABLE_LOCALE_SUPPORT
#endif

#if ENABLE_FEATURE_HTTPD_ENCODE_URL_STR
#endif

#if ENABLE_FEATURE_HTTPD_AUTH_MD5
#endif

#if ENABLE_FEATURE_HTTPD_SETUID
#endif

#if ENABLE_FEATURE_IPV6
#endif

#if ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV
#endif


/*
 * check-name: coverage analysis for busybox httpd
 * check-command: undertaker -j coverage -O stdout -m  busybox-httpd.model $file
 * check-output-start
I: Set configuration space regex to '^(ENABLE_|CONFIG_)[^ ]*$'
I: loaded rsf model for busybox-httpd.model
I: Using busybox-httpd.model as primary model
I: Found 4 assignments
I: In all assignments the following symbols are equally set
CONFIG_FEATURE_HTTPD_AUTH_MD5=1
CONFIG_FEATURE_HTTPD_BASIC_AUTH=1
CONFIG_FEATURE_HTTPD_ENCODE_URL_STR=1
CONFIG_FEATURE_HTTPD_ERROR_PAGES=1
CONFIG_FEATURE_HTTPD_PROXY=1
CONFIG_FEATURE_HTTPD_SETUID=1
CONFIG_FEATURE_HTTPD_USE_SENDFILE=1
CONFIG_FEATURE_IPV6=1
CONFIG_HTTPD=1
CONFIG_LOCALE_SUPPORT=1
ENABLE_FEATURE_HTTPD_AUTH_MD5=1
ENABLE_FEATURE_HTTPD_BASIC_AUTH=1
ENABLE_FEATURE_HTTPD_ENCODE_URL_STR=1
ENABLE_FEATURE_HTTPD_ERROR_PAGES=1
ENABLE_FEATURE_HTTPD_PROXY=1
ENABLE_FEATURE_HTTPD_SETUID=1
ENABLE_FEATURE_HTTPD_USE_SENDFILE=1
ENABLE_FEATURE_IPV6=1
ENABLE_HTTPD=1
ENABLE_LOCALE_SUPPORT=1
I: All differences in the assignments
I: Config 0
CONFIG_FEATURE_HTTPD_CGI=1
CONFIG_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
CONFIG_FEATURE_HTTPD_GZIP=1
CONFIG_FEATURE_HTTPD_RANGES=1
CONFIG_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
ENABLE_FEATURE_HTTPD_CGI=1
ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
ENABLE_FEATURE_HTTPD_GZIP=1
ENABLE_FEATURE_HTTPD_RANGES=1
ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
I: Config 1
CONFIG_FEATURE_HTTPD_CGI=1
CONFIG_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
CONFIG_FEATURE_HTTPD_GZIP=1
CONFIG_FEATURE_HTTPD_RANGES=0
CONFIG_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
ENABLE_FEATURE_HTTPD_CGI=1
ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
ENABLE_FEATURE_HTTPD_GZIP=1
ENABLE_FEATURE_HTTPD_RANGES=0
ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
I: Config 2
CONFIG_FEATURE_HTTPD_CGI=1
CONFIG_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
CONFIG_FEATURE_HTTPD_GZIP=0
CONFIG_FEATURE_HTTPD_RANGES=0
CONFIG_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
ENABLE_FEATURE_HTTPD_CGI=1
ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=1
ENABLE_FEATURE_HTTPD_GZIP=0
ENABLE_FEATURE_HTTPD_RANGES=0
ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=1
I: Config 3
CONFIG_FEATURE_HTTPD_CGI=0
CONFIG_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=0
CONFIG_FEATURE_HTTPD_GZIP=0
CONFIG_FEATURE_HTTPD_RANGES=0
CONFIG_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=0
ENABLE_FEATURE_HTTPD_CGI=0
ENABLE_FEATURE_HTTPD_CONFIG_WITH_SCRIPT_INTERPR=0
ENABLE_FEATURE_HTTPD_GZIP=0
ENABLE_FEATURE_HTTPD_RANGES=0
ENABLE_FEATURE_HTTPD_SET_REMOTE_PORT_TO_ENV=0
 * check-output-end
 */
