/*
 * check-name: find interesting items for symbol
 * check-command: undertaker -j interesting -m preconditions.model CONFIG_LEVEL_C_B
 * check-output-start
I: loaded rsf model for preconditions
I: Using preconditions as primary model
CONFIG_LEVEL_C_B !CONFIG_I_DONT_GIVE_A_BLOODY_HELL_PRECONDITION CONFIG_NOT_MISSING CONFIG_TOPLEVEL_C
 * check-output-end
 */
