#ifdef CONFIG_BAR

#else

#endif

/*
 * check-name: Check for file-based presence conditions
 * check-command: undertaker -m file-presence-conditions.model $file
 * check-output-start
I: Using file-presence-conditions.model as primary model
I: loaded rsf model for file-presence-conditions.model
I: creating inference.c.B1.kconfig.globally.dead
 * check-output-end
 */

