#if CONFIG_BAR
#endif

#if BARFOO
#endif

#ifdef CONFIG_BAR
#else
#endif

#ifdef HOLLA
#elif CONFIG_BAR
#endif

#ifdef A
#ifdef B
#if CONFIG_C
#endif
#if D
#endif
#endif
#endif

#if A
#if B
#if CONFIG_FOO
#elif HALLO
#endif
#endif
#endif

#if C
#if D
#endif
#endif
