"""
Attributes:

   default_architecture   -- the default architecture to use
   prefer_32bit  -- If running on an amd64 64bit host, prefer configuring and compiling for 32bit.
   kernelversion -- normally determined automatically, but can be overridden here
"""

default_architecture = "x86"

prefer_32bit = True

kernelversion = None

